@@ -95,7 +95,7 @@ typedef struct
   SQLULEN       ArraySize;
   SQLUSMALLINT *ArrayStatusPtr;
   SQLULEN      *BindOffsetPtr;
-  SQLINTEGER    BindType;
+  SQLULEN       BindType;
   SQLSMALLINT   Count;
   SQLULEN      *RowsProcessedPtr;
   /* Header end */
@@ -400,7 +400,6 @@ struct st_ma_odbc_connection
   HWND QuietMode;
   SQLUINTEGER Trace;
   char *TraceFile;
-  char *TranslateLib;
   SQLINTEGER TxnIsolation;
   SQLINTEGER CursorCount;
 };
@@ -485,13 +484,6 @@ SQLRETURN MA_SQLGetConnectAttr(SQLHDBC ConnectionHandle,
     SQLINTEGER BufferLength,
     SQLINTEGER *StringLengthPtr);
 
-SQLRETURN MA_SQLGetData(SQLHSTMT StatementHandle,
-    SQLUSMALLINT Col_or_Param_Num,
-    SQLSMALLINT TargetType,
-    SQLPOINTER TargetValuePtr,
-    SQLLEN BufferLength,
-    SQLLEN *StrLen_or_IndPtr);
-
 SQLRETURN MA_SQLPrepare(MADB_Stmt *Stmt,
     SQLCHAR *StatementText,
     SQLINTEGER TextLength);
@@ -530,6 +522,12 @@ SQLRETURN MA_SQLSetStmtAttr(SQLHSTMT StatementHandle,
 SQLRETURN MA_SQLAllocConnect(SQLHANDLE InputHandle,
                              SQLHANDLE *OutputHandlePtr);
 
+SQLRETURN MADB_GetBookmark(MADB_Stmt  *StatementHandle,
+                           SQLSMALLINT TargetType,
+                           SQLPOINTER  TargetValuePtr,
+                           SQLLEN      BufferLength,
+                           SQLLEN     *StrLen_or_IndPtr);
+
 SQLRETURN MADB_StmtColAttr(MADB_Stmt *Stmt, SQLUSMALLINT ColumnNumber, SQLUSMALLINT FieldIdentifier, SQLPOINTER CharacterAttributePtr,
              SQLSMALLINT BufferLength, SQLSMALLINT *StringLengthPtr, SQLLEN *NumericAttributePtr, my_bool IsWchar);
 
