@@ -1,5 +1,5 @@
 /************************************************************************************
-   Copyright (C) 2013 SkySQL AB
+   Copyright (C) 2013,2016 MariaDB Corporation AB
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -42,8 +42,6 @@ MADB_QUERY *MADB_Tokenize(const char *Stmt)
   const char *End= (char *)Stmt + strlen(Stmt);
   const char *Pos;
   const char *Start= Stmt;
-  const char *p= Stmt;
-  int Tokens= 0;
 
   MADB_QUERY *Query= (MADB_QUERY *)MADB_CALLOC(sizeof(MADB_QUERY));
   init_dynamic_array(&Query->tokens, sizeof(unsigned int), 20, 20);
@@ -59,8 +57,10 @@ MADB_QUERY *MADB_Tokenize(const char *Stmt)
 
 void MADB_FreeTokens(MADB_QUERY *Query)
 {
-  if (!Query)
+  if (Query == NULL)
+  {
     return;
+  }
   delete_dynamic(&Query->tokens);
   MADB_FREE(Query);
 }
@@ -145,26 +145,47 @@ const char * MADB_FindParamPlaceholder(MADB_Stmt *Stmt)
 
 enum enum_madb_query_type MADB_GetQueryType(MADB_Stmt *Stmt)
 {
-  if (_strnicmp(Stmt->StmtString, "SELECT", 6) == 0)
+  char *p= Stmt->StmtString;
+
+  while (*p && !isalpha(*p))
+    ++p;
+
+  if (_strnicmp(p, "SELECT", 6) == 0)
   {
     return MADB_QUERY_SELECT;
   }
-  if (_strnicmp(Stmt->StmtString, "UPDATE", 6) == 0)
+  if (_strnicmp(p, "UPDATE", 6) == 0)
   {
     return MADB_QUERY_UPDATE;
   }
-  if (_strnicmp(Stmt->StmtString, "DELETE", 6) == 0)
+  if (_strnicmp(p, "DELETE", 6) == 0)
   {
     return MADB_QUERY_DELETE;
   }
-  if (_strnicmp(Stmt->StmtString, "CALL", 4) == 0)
+  if (_strnicmp(p, "CALL", 4) == 0)
   {
     return MADB_QUERY_CALL;
   }
-  if (_strnicmp(Stmt->StmtString, "SHOW", 4) == 0)
+  if (_strnicmp(p, "SHOW", 4) == 0)
   {
     return MADB_QUERY_SHOW;
   }
+  if (_strnicmp(p, "ANALYZE", 7) == 0)
+  {
+    return MADB_QUERY_ANALYZE;
+  }
+  if (_strnicmp(p, "EXPLAIN", 7) == 0)
+  {
+    return MADB_QUERY_EXPLAIN;
+  }
+  if (_strnicmp(p, "CHECK", 5) == 0)
+  {
+    return MADB_QUERY_CHECK;
+  }
+  if (_strnicmp(p, "EXECUTE", 7) == 0)
+  {
+    return MADB_QUERY_EXECUTE;
+  }
 
   return MADB_QUERY_NO_RESULT;
 }
