@@ -85,7 +85,7 @@ my_bool MADB_DynStrAppendQuoted(DYNAMIC_STRING *DynString, char *String)
 
 my_bool MADB_DynStrUpdateSet(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
 {
-  int             i, IgnoredColumns= 0, Count= 0;
+  int             i, IgnoredColumns= 0;
   MADB_DescRecord *Record;
 
   if (dynstr_append(DynString, " SET "))
@@ -99,7 +99,8 @@ my_bool MADB_DynStrUpdateSet(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
     SQLLEN *IndicatorPtr= NULL;
     Record= MADB_DescGetInternalRecord(Stmt->Ard, i, MADB_DESC_READ);
     if (Record->IndicatorPtr)
-      IndicatorPtr= (SQLLEN *)GetBindOffset(Stmt->Ard, Record, Record->IndicatorPtr, MAX(0, Stmt->DaeRowNumber-1), Record->OctetLength);
+      IndicatorPtr= (SQLLEN *)GetBindOffset(Stmt->Ard, Record, Record->IndicatorPtr, Stmt->DaeRowNumber > 1 ? Stmt->DaeRowNumber-1 : 0,
+                                            sizeof(SQLLEN)/*Record->OctetLength*/);
     if ((IndicatorPtr && *IndicatorPtr == SQL_COLUMN_IGNORE) || !Record->inUse)
     {
       IgnoredColumns++;
@@ -144,18 +145,15 @@ my_bool MADB_DynStrInsertSet(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString)
     return TRUE;
   }
 
+  /* We use only columns, that have been bound, and are not IGNORED */
   for (i= 0; i < MADB_STMT_COLUMN_COUNT(Stmt); i++)
   {
-    SQLINTEGER *IndicatorPtr= NULL;
     Record= MADB_DescGetInternalRecord(Stmt->Ard, i, MADB_DESC_READ);
-    if (Record->IndicatorPtr)
-      IndicatorPtr= (SQLINTEGER *)GetBindOffset(Stmt->Ard, Record, Record->IndicatorPtr, MAX(0, Stmt->DaeRowNumber-1), Record->OctetLength);
     
-    /* We prepare query only once, different paramsets may have different SQL_COLUMN_IGNORE */
-    /*if (IndicatorPtr && *IndicatorPtr == SQL_COLUMN_IGNORE)
+    if (!Record->inUse || MADB_ColumnIgnoredInAllRows(Stmt->Ard, Record) == TRUE)
     {
       continue;
-    }*/
+    }
 
     if ((NeedComma) && 
         (dynstr_append(DynString, ",") || dynstr_append(&ColVals, ",")))
@@ -265,7 +263,7 @@ my_bool MADB_DynStrGetWhere(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString, char *Ta
       }
       else
       { 
-        if (!SQL_SUCCEEDED(Stmt->Methods->GetData(Stmt, i+1, SQL_C_CHAR, NULL, 0, &StrLength)))
+        if (!SQL_SUCCEEDED(Stmt->Methods->GetData(Stmt, i+1, SQL_C_CHAR, NULL, 0, &StrLength, TRUE)))
         {
           MADB_FREE(Column);
           return TRUE;
@@ -278,7 +276,7 @@ my_bool MADB_DynStrGetWhere(MADB_Stmt *Stmt, DYNAMIC_STRING *DynString, char *Ta
         else
         {
           Column= MADB_CALLOC(StrLength + 1);
-          Stmt->Methods->GetData(Stmt,i+1, SQL_C_CHAR, Column, StrLength + 1, NULL);
+          Stmt->Methods->GetData(Stmt,i+1, SQL_C_CHAR, Column, StrLength + 1, NULL, TRUE);
           if (dynstr_append(DynString, "= '") ||
                  dynstr_append(DynString, Column) ||
                  dynstr_append(DynString, "'"))
@@ -421,9 +419,28 @@ my_bool MADB_ValidateStmt(char *StmtStr)
 }
 
 
+char *MADB_ToLower(const char *src, char *buff, size_t buff_size)
+{
+  size_t i= 0;
+
+  if (buff_size > 0)
+  {
+    while (*src && i < buff_size)
+    {
+      buff[i++]= tolower(*src++);
+    }
+
+    buff[i == buff_size ? i - 1 : i]= '\0';
+  }
+  return buff;
+}
+
+
 int InitClientCharset(Client_Charset *cc, const char * name)
 {
-  cc->cs_info= mysql_get_charset_by_name(name);
+  /* There is no legal charset names longer than 31 chars */
+  char lowered[32];
+  cc->cs_info= mysql_get_charset_by_name(MADB_ToLower(name, lowered, sizeof(lowered)));
 
   if (cc->cs_info == NULL)
   {
@@ -476,7 +493,7 @@ SQLLEN MbstrOctetLen(char *str, SQLLEN *CharLen, CHARSET_INFO *cs)
     {
       while (inChars > 0 || inChars < 0 && *str)
       {
-        result+= cs->mb_charlen(*str);
+        result+= cs->mb_charlen(0 + *str);
         --inChars;
         str+= cs->mb_charlen(*str);
       }
